// mesh.h
// By Ross Light, 2012.
// Released as public domain. Don't be evil. :)

#ifndef _MESH_H_
#define _MESH_H_

#include <string>
#include <vector>

#ifdef __APPLE__
#include <OpenGL/gl.h>
#endif
#ifdef __unix__
#include <GL/gl.h>
#endif
#include "../glm/glm.hpp"

class ObjMesh;
struct Face;

typedef std::vector<ObjMesh> ObjMeshVector;

class Model
{
    friend class ObjMesh;
public:
    void load(std::istream& is);
    const ObjMeshVector& objMeshes() const { return mObjMeshes; }
    int materialCount;

private:
    ObjMeshVector mObjMeshes;
    std::vector<glm::vec3> mVertices;
    std::vector<glm::vec2> mUVs;
    std::vector<glm::vec3> mNormals;
    
};

class ObjMesh
{
    friend class Model;
public:
    const std::string& name() const { return mName; }
    int makeVBO(GLuint* indexHandle, GLuint* posHandle, GLuint* uvHandle, GLuint* normalHandle) const;
    std::vector<Face> mFaces;

private:
    ObjMesh(const Model& model, const std::string& name);

    const Model* mModel;
    std::string mName;
};

struct Face
{
    std::string mtl;
    struct Vertex
    {
        int mVertex;
        int mUV;
        int mNormal;
    };

    Vertex V[3];
};

std::istream& operator>>(std::istream& is, Face::Vertex& vert);
bool operator<(const Face::Vertex& a, const Face::Vertex& b);
bool operator==(const Face::Vertex& a, const Face::Vertex& b);
bool operator!=(const Face::Vertex& a, const Face::Vertex& b);

#endif
