/*
 *  CPE 474 lab 0 - modern graphics test bed
 *  draws a partial cube using a VBO and IBO
 *  glut/OpenGL/GLSL application
 *  Uses glm and local matrix stack
 *  to handle matrix transforms for a view matrix, projection matrix and
 *  model transform matrix
 *
 *  zwood 9/12
 *  Copyright 2012 Cal Poly. All rights reserved.
 *
 *****************************************************************************/
#ifdef __APPLE__
#include "GLUT/glut.h"
#include <OPENGL/gl.h>
#endif
#ifdef __unix__
#include <GL/glut.h>
#endif
#include <map>
#include <sstream>
#include <stdlib.h>
#include <stdio.h>
#include "glm/glm.hpp"
#include "glm/gtc/matrix_transform.hpp" //perspective, trans etc
#include "glm/gtc/type_ptr.hpp" //value_ptr
#include "GLSL_helper.h"
#include "MStackHelp.h"
#include "CoordinateTransforms.h"
#include "CMeshLoaderSimple.h"
#include "GeometryCreator.h"
#include "OBJ_fromRL/mesh.h"
#include <fstream>

float mouseDragStartX = 0;
float mouseDragStartY = 0;
float mouseDragEndX = 0;
float mouseDragEndY = 0;
float mouseTransX = 0;
float mouseTransY = 0;
float mouseUpX = 0;
float mouseUpY = 0;
float mouseDownX = 0;
float mouseDownY = 0;

GLint h_uTexUnit;

int vertexCount; 
using namespace std;
using namespace glm;
GLuint positions, uvs, normals, ibo;

//flag and ID to toggle on and off the shader
int shade = 1;
int ShadeProg;
int currentLevel = 1;
//Handles to the shader data
GLint h_aPosition;
GLint h_uColor;
GLint h_aNormal;
GLint h_aUV;
GLint h_uModelMatrix;
GLint h_uViewMatrix;
GLint h_uProjMatrix;
GLint h_uLightVector;
GLint h_aCurTex;
GLint h_aTexCoord;
GLint h_uHasATexture;

GLuint triBuffObj, colBuffObj, flatNormalsBufferObject, phongBufferObject;
GLuint GameGuyTriBuffObj, GameGuyColBuffObj, GameGuyPhongBufferObject;
GLuint CubeBuffObj, CIndxBuffObj, GrndBuffObj, GIndxBuffObj, NormalBuffObj, GroundNormalBuffObj, RIndxBuffObj;
GLuint SkyBuffObj, SkyIndxBuffObj, ShadowIndxBuffObj, ShadowBuffObj;
GLuint PortalBuffObj, PortalIndxBuffObj;
GLuint BunnyTriBuffObj, BunnyColBuffObj, BunnyPhongBufferObject;

GLuint GTexBuffObj, RoadTexBuffObj, SkyTexBuffObj, PortalTexBuffObj;


int g_CiboLen, g_GiboLen, g_RiboLen, g_SkyIBOLength, g_ShadowIBOLength, g_PortalIBOLength;

/* globals to control positioning and window size */
static float g_width, g_height;
float g_tx = 0;
float g_ty = 0;
float g_trans = -5.5;
float g_trans_x = 0;
float g_trans_z = 0;
float g_angle = 0;
float lastMouseTranslateX = 0;
float lastMouseTranslateY = 0;
float houseCoords[20 * 2];
float treeCoords[40 * 2];
float peopleCoords[20 * 2];
float houseColors[20 * 3];

static const float g_groundY = -0.501;      // y coordinate of the ground
static const float g_groundSize = 50.0;   // half the ground length

struct Material {
    vec3 aColor;
    vec3 dColor;
    vec3 sColor;
    float shine;
};

//declare a matrix stack
RenderingHelper ModelTrans;
vec3 defaultEyePos = vec3(0, 1.5, -g_groundSize);
vec3 defaultPlayerPos = vec3(0, 0, -g_groundSize);
vec3 defaultLookAtVector = vec3(0, 1.5, 1 - g_groundSize);
vec3 defaultCarPos = glm::vec3(0, g_ty + 2.0, -3 * g_groundSize);

vec3 eyePos = vec3(0, 1.5, -g_groundSize);
vec3 lookAtVector = vec3(0, 1.5, 1 - g_groundSize);
vec3 carPos = glm::vec3(0, g_ty + 2.0, -3 * g_groundSize);

vec3 upVector = vec3(0, 1, 0);
vec3 lightPosition = vec3(0.75, 1.0, -1.0);
float pitchAngle = 0.0;
float yawAngle = 3.14/2.0;
float heightForMouse = 3.14;
float widthForMouse = 3.14;
float speedFactor = 0.5;
float mouseSpeedFactor = 0.05;
int TriangleCount;
int GameGuyTriangleCount;
int BunnyTriangleCount;


Mesh * treeTop;
Mesh * treeTrunk;
Mesh * scoreTally;
Mesh * speedMeterBackground;
Mesh * speedMeter;

Mesh * speedArrow;
Mesh * statusBar;
// Lighting
GLint h_uLightColor;
GLint h_uMatAmb, h_uMatDif, h_uMatSpec, h_uMatShine;


// Road
float g_roadWidth = 7;
float g_roadLength = g_groundSize;
int g_RoadIBOlen;
GLuint RoadBuffObj;
GLuint RoadNormalBuffObj;
GLuint RoadIndxBuffObj;
bool gamePaused = false;
vec3 playerPosition;
vec3 gazeVector;
vec3 wVector;
vec3 uVector;
vec3 g_light_trans = glm::vec3(0, 10.0, 40.0);
vec3 scoreTallyDefaultPosition;
vec3 speedMeterPosition;

/*data structure for the image used for  texture mapping */
typedef struct Image {
    unsigned long sizeX;
    unsigned long sizeY;
    char *data;
} Image;

Image *TextureImage;
Model model;

typedef struct RGB {
    GLubyte r;
    GLubyte g;
    GLubyte b;
} RGB;

RGB myimage[64][64];
RGB* g_pixel;
//forward declaration of image loading and texture set-up code
int ImageLoad(char *filename, Image *image);
GLvoid LoadTexture(char* image_file, int tex_id);

/* projection matrix - do not change - sets matrix in shader */
void SetProjectionMatrix() {
    glm::mat4 Projection = glm::perspective(80.0f, (float)g_width/g_height, 0.1f, 100.f);
    safe_glUniformMatrix4fv(h_uProjMatrix, glm::value_ptr(Projection));
}
/* model transforms */
void SetLightModel() {
    glm::mat4 Trans = glm::translate( glm::mat4(1.0f), g_light_trans);
    glm::mat4 Scale = glm::scale( glm::mat4(1.0f), glm::vec3(0.5));
    
    glm::mat4 Model = Trans * Scale;
    safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(Model));
}
/* helper function to set up material for shading */
void SetMaterial(int i) {
    
    glUseProgram(ShadeProg);
    switch (i) {
        case 0:
            safe_glUniform3f(h_uMatAmb, 0.882353, 0.470588, 0.235294);
            safe_glUniform3f(h_uMatDif, 0.882353, 0.470588, 0.235294);
            safe_glUniform3f(h_uMatSpec, 0.882353, 0.470588, 0.235294);
            safe_glUniform1f(h_uMatShine, 0.0);
            break;
        case 1:
            safe_glUniform3f(h_uMatAmb, 0.105882, 0.611765, 0.952941);
            safe_glUniform3f(h_uMatDif, 0.105882, 0.611765, 0.952941);
            safe_glUniform3f(h_uMatSpec, 0.105882, 0.611765, 0.952941);
            safe_glUniform1f(h_uMatShine, 0.0);
            break;
        case 2:
            safe_glUniform3f(h_uMatAmb,  0.494118, 0.796079, 0.454902);
            safe_glUniform3f(h_uMatDif,  0.0, 0.0, 0.0);
            safe_glUniform3f(h_uMatSpec, 0.0, 0.0, 0.0);
            safe_glUniform1f(h_uMatShine, 0.0);
            break;
        case 3:
            safe_glUniform3f(h_uMatAmb, 0.768628, 0.447059, 0.447059);
            safe_glUniform3f(h_uMatDif, 0.266667, 0.301961, 0.415686);
            safe_glUniform3f(h_uMatSpec, 1.0, 1.0, 1.0);
            safe_glUniform1f(h_uMatShine, 100.0);
            break;
        case 4:
            safe_glUniform3f(h_uMatAmb,	0.02,	0.02,	0.1);
            safe_glUniform3f(h_uMatDif,	0.0,	0.08,	0.5);
            safe_glUniform3f(h_uMatSpec,	0.14,	0.14,	0.4);
            safe_glUniform1f(h_uMatShine,	120.0);
            break;
        case 5:
            safe_glUniform3f(h_uMatAmb,	0.1, 0.1, 0.1);
            safe_glUniform3f(h_uMatDif, 0.1, 0.1, 0.1);
            safe_glUniform3f(h_uMatSpec, 0.1, 0.1, 0.1);
            safe_glUniform1f(h_uMatShine, 0.0);
            break;
        case 6:
            safe_glUniform3f(h_uMatAmb,	0.694118, 0.266667, 0.921569);
            safe_glUniform3f(h_uMatDif, 0.694118, 0.266667, 0.921569);
            safe_glUniform3f(h_uMatSpec, 0.694118, 0.266667, 0.921569);
            safe_glUniform1f(h_uMatShine, 0.0);
            break;
        case 7:
            safe_glUniform3f(h_uMatAmb,	0.376471, 0.678431, 0.921569);
            safe_glUniform3f(h_uMatDif, 0.376471, 0.678431, 0.921569);
            safe_glUniform3f(h_uMatSpec, 0.376471, 0.678431, 0.921569);
            safe_glUniform1f(h_uMatShine, 0.0);
            break;
        case 8:
            safe_glUniform3f(h_uMatAmb,	0.19225,0.19225, 0.19225);
            safe_glUniform3f(h_uMatDif,	0.50754,0.50754,0.50754);
            safe_glUniform3f(h_uMatSpec, 0.508273,0.508273,0.508273);
            safe_glUniform1f(h_uMatShine, 51.2);
            break;
        case 9:
            safe_glUniform3f(h_uMatAmb, 0.1, 0.1, 0.1);
            safe_glUniform3f(h_uMatDif,  0.1, 0.1, 0.1);
            safe_glUniform3f(h_uMatSpec,  0.2, 0.2, 0.2);
            safe_glUniform1f(h_uMatShine, 50.0);
            break;
        case 10:
            safe_glUniform3f(h_uMatAmb, 0.329410, 0.223529, 0.027451);
            safe_glUniform3f(h_uMatDif,  0.780392, 0.568627, 0.113725);
            safe_glUniform3f(h_uMatSpec,  0.992157, 0.941176, 0.807843);
            safe_glUniform1f(h_uMatShine, 27.8974);
            break;
        // Tree trunk material 0.7961 0.6196 0.4745
        case 11:
            safe_glUniform3f(h_uMatAmb, 0.7961, 0.6196, 0.4745);
            safe_glUniform3f(h_uMatDif,  0.0, 0.0, 0.0);
            safe_glUniform3f(h_uMatSpec, 0.0, 0.0, 0.0);
            safe_glUniform1f(h_uMatShine, 0.0);
            break;
        // Tree top material 0.5804 0.7059 0.4353
        case 12:
            safe_glUniform3f(h_uMatAmb, 0.5804, 0.7059, 0.4353);
            safe_glUniform3f(h_uMatDif,  0.0, 0.0, 0.0);
            safe_glUniform3f(h_uMatSpec, 0.0, 0.0, 0.0);
            safe_glUniform1f(h_uMatShine, 0.0);
            break;
            // Speed meter
        case 13:
            safe_glUniform3f(h_uMatAmb, 1.0, 1.0, 1.0);
            safe_glUniform3f(h_uMatDif,  0.0, 0.0, 0.0);
            safe_glUniform3f(h_uMatSpec, 0.0, 0.0, 0.0);
            safe_glUniform1f(h_uMatShine, 0.0);
            break;
            // Speed arrow
        case 14:
            safe_glUniform3f(h_uMatAmb, 0.4980, 0.0118, 0.0314);
            safe_glUniform3f(h_uMatDif,  0.0, 0.0, 0.0);
            safe_glUniform3f(h_uMatSpec, 0.0, 0.0, 0.0);
            safe_glUniform1f(h_uMatShine, 0.0);
            break;
    }
}

int getRandomNumberBetween(int max, int min)
{
    return rand() % (max-min) + min;
}

/* camera controls - do not change - sets matrix in shader */
void SetView() {
    
    //glm::mat4 translation = glm::translate( glm::mat4(1.0f), glm::vec3(g_trans_x, 0, g_trans_z));
    eyePos.y = 1.5;
    lookAtVector.y = 1.5;
    glm::mat4 view = glm::lookAt(eyePos, lookAtVector, upVector);
    
    //glm::mat4 total = translation * view;
    safe_glUniformMatrix4fv(h_uViewMatrix, glm::value_ptr(view));
}

/* model transforms - do not change - sets matrix in shader */
void SetModel() {
    safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(ModelTrans.modelViewMatrix));
}

void InitTreeCoords()
{
    for (int treeNum = 0; treeNum < 40; treeNum++)
    {
        treeCoords[treeNum] = getRandomNumberBetween(g_groundSize, -g_groundSize);
        treeCoords[treeNum + 1] = getRandomNumberBetween(g_roadLength, -g_roadLength);
    }
}
void loadLevel(std::istream& is)
{
    int currentObstacle = 0;
    while (is.good()) {
        std::string line;
        std::getline(is, line);
        if (line.empty() || line[0] == '#') {
            continue;
        }
        std::istringstream lineStream(line);
        
        std::string cmd;
        lineStream >> cmd;
        if (cmd == "v")
        {
            float x, z;
            lineStream >> x >> z;
            houseCoords[currentObstacle] = x;
            houseCoords[currentObstacle + 1] = z;
            currentObstacle = currentObstacle + 2;
            
        }
    }
}


void InitHouseCoords()
{
    // First house (level 1)
    houseCoords[0] = 0;     // x
    houseCoords[1] = 0;     // z
    
    // Second house (level 2)
    houseCoords[2] = 2;     // x
    houseCoords[3] = 5;     // z
    
    // Third house (level 3)
    houseCoords[4] = 2;     // x
    houseCoords[5] = 3;     // z
    
    // Fourth house (level 4)
    houseCoords[6] = -1;     // x
    houseCoords[7] = -5;     // z
    
    // Fifth house (level 5)
    houseCoords[8] = 0;     // x
    houseCoords[9] = -1;     // z
    
    // Sixth house (level 6)
    houseCoords[10] = 7;     // x
    houseCoords[11] = -12;     // z
    
    houseCoords[12] = 10;     // x
    houseCoords[13] = -20;     // z
    
    houseCoords[14] = 7;     // x
    houseCoords[15] = -12;     // z
    
    // Second house (level 2)
    houseCoords[16] = 2;     // x
    houseCoords[17] = -10;     // z
    
    // Third house (level 3)
    houseCoords[18] = -3;     // x
    houseCoords[19] = 4;     // z
    
    // Fourth house (level 4)
    houseCoords[20] = -1;     // x
    houseCoords[21] = -20;     // z
    
    // Fifth house (level 5)
    houseCoords[22] = 0;     // x
    houseCoords[23] = 12;     // z
    
    // Sixth house (level 6)
    houseCoords[24] = 3;     // x
    houseCoords[25] = 7;     // z
    
    houseCoords[26] = -8;     // x
    houseCoords[27] = 1;     // z
    
    houseCoords[28] = -3;     // x
    houseCoords[29] = 1;     // z
    
    houseCoords[30] = -2;     // x
    houseCoords[31] = 7;     // z
    
    houseCoords[32] = 1;     // x
    houseCoords[33] = 8;     // z
    
    houseCoords[34] = 10;     // x
    houseCoords[35] = 10;     // z
    
    houseCoords[36] = -10;     // x
    houseCoords[37] = -11;     // z
    
    houseCoords[38] = -3;     // x
    houseCoords[39] = 4;     // z
    
}

static void initHouseShadow()
{
    float ShadowPos[] =
    {
        -0.5, g_groundY + 0.27f, -0.5 - 1.0,
        -0.5, g_groundY + 0.27f, 0.5 - 1.0,
        0.5, g_groundY + 0.27f, -0.5 - 1.0,
        0.5, g_groundY + 0.27f, 0.5 - 1.0
    };
    
    unsigned short idx[] = {0, 1, 2, 1, 2, 3};
    g_ShadowIBOLength = 6;
    glGenBuffers(1, &ShadowBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, ShadowBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(ShadowPos), ShadowPos, GL_STATIC_DRAW);
    
    glGenBuffers(1, &ShadowIndxBuffObj);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, ShadowIndxBuffObj);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(idx), idx, GL_STATIC_DRAW);
}

static void initPortal()
{
    float PortalPos[] = {
        -g_groundSize, g_groundY, g_groundSize,
        -g_groundSize, g_groundSize, g_groundSize,
        g_groundSize, g_groundSize, g_groundSize,
        g_groundSize, g_groundY, g_groundSize
    };
    
    float PortalTex[] = {
        0, 0,
        0, 1,
        1, 1,
        1, 0
    };
    
    unsigned short idx[] = {0, 1, 2,  3, 2, 0};
    
    g_PortalIBOLength = 6;
    glGenBuffers(1, &PortalBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, PortalBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(PortalPos), PortalPos, GL_STATIC_DRAW);
    
    glGenBuffers(1, &PortalIndxBuffObj);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, PortalIndxBuffObj);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(idx), idx, GL_STATIC_DRAW);
    
    glGenBuffers(1, &PortalTexBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, PortalTexBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(PortalTex), PortalTex, GL_STATIC_DRAW);

}
static void initSky()
{
    float SkyPos[] = {
        -g_groundSize, g_groundY, -g_groundSize, // left 0
        -g_groundSize, g_groundY, g_groundSize,
        -g_groundSize, g_groundSize, -g_groundSize,
        -g_groundSize, g_groundSize, g_groundSize,
        -g_groundSize, g_groundY, g_groundSize, // front 4
        -g_groundSize, g_groundSize, g_groundSize,
        g_groundSize, g_groundSize, g_groundSize,
        g_groundSize, g_groundY, g_groundSize,
        g_groundSize, g_groundY, g_groundSize, // right 8
        g_groundSize, g_groundSize, g_groundSize,
        g_groundSize, g_groundSize, -g_groundSize,
        g_groundSize, g_groundY, -g_groundSize,
        g_groundSize, g_groundSize, -g_groundSize, // back
        g_groundSize, g_groundY, -g_groundSize,
        -g_groundSize, g_groundSize, -g_groundSize,
        -g_groundSize, g_groundY, -g_groundSize,
        g_groundSize, g_groundSize, -g_groundSize, // top
        g_groundSize, g_groundSize, g_groundSize,
        -g_groundSize, g_groundSize, -g_groundSize,
        -g_groundSize, g_groundSize, g_groundSize
        
    };
    
    float SkyTex[] =
    {
        0, 0,
        0, 1,
        1, 1,
        1, 0,
        0, 0,
        0, 0,
        0, 0,
        0, 0,
        0, 0,
        0, 1,
        1, 1,
        1, 0,
        0, 0,
        0, 1,
        1, 1,
        1, 0,
        0, 0,
        0, 1,
        1, 1,
        1, 0
    };

    
    unsigned short idx[] = {0, 1, 2,  1, 2, 3,  4, 5, 7,  6, 5, 7,  8, 9, 10,  11, 8, 10,  12, 13, 14,  15, 13, 14,  16, 17, 18,  19, 18, 17};
    
    g_SkyIBOLength = 30;
    glGenBuffers(1, &SkyBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, SkyBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(SkyPos), SkyPos, GL_STATIC_DRAW);
    
    glGenBuffers(1, &SkyIndxBuffObj);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, SkyIndxBuffObj);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(idx), idx, GL_STATIC_DRAW);
    
    glGenBuffers(1, &SkyTexBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, SkyTexBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(SkyTex), SkyTex, GL_STATIC_DRAW);


}
/* intialize ground data */
static void initGround() {
    
    // A x-z plane at y = g_groundY of dimension [-g_groundSize, g_groundSize]^2
    float GrndPos[] = {
        -g_groundSize, g_groundY, -g_groundSize, // bottom 0
        -g_groundSize, g_groundY,  g_groundSize,
        g_groundSize, g_groundY,  g_groundSize,
        g_groundSize, g_groundY, -g_groundSize
    };
    
    float GroundNormal[] =
    {
        0, 1.0, 0,
        0, 1.0, 0,
        0, 1.0, 0,
        0, 1.0, 0
    };
    
    float GrndTex[] =
    {
        0, 0,
        0, 5,
        5, 5,
        5, 0
    };
    
    unsigned short idx[] = {0, 1, 2, 0, 2, 3};
    
    g_GiboLen = 6;
    glGenBuffers(1, &GrndBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, GrndBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(GrndPos), GrndPos, GL_STATIC_DRAW);
    
    
    glGenBuffers(1, &GroundNormalBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, GroundNormalBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(GroundNormal), GroundNormal, GL_STATIC_DRAW);
    
    glGenBuffers(1, &GIndxBuffObj);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, GIndxBuffObj);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(idx), idx, GL_STATIC_DRAW);
    
    glGenBuffers(1, &GTexBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, GTexBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(GrndTex), GrndTex, GL_STATIC_DRAW);
}

/* intialize road data */
static void initRoad() {
    
    // A x-z plane at y = g_groundY of dimension [-g_groundSize, g_groundSize]^2
    float RoadPos[] = {
        -g_roadWidth, g_groundY + 0.1f, -g_roadLength, // bottom 0
        -g_roadWidth, g_groundY + 0.1f,  g_roadLength,
        g_roadWidth, g_groundY + 0.1f,  g_roadLength,
        g_roadWidth, g_groundY + 0.1f, -g_roadLength
    };
    
    float RoadNormal[] =
    {
        0, 1.0, 0,
        0, 1.0, 0,
        0, 1.0, 0,
        0, 1.0, 0
    };
    
    float RoadTex[] =
    {
        0, 0,
        0, 2,
        2, 2,
        2, 0
    };
    
    unsigned short idx[] = {0, 1, 2, 0, 2, 3};
    
    g_RoadIBOlen = 6;
    glGenBuffers(1, &RoadBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, RoadBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(RoadPos), RoadPos, GL_STATIC_DRAW);
    
    
    glGenBuffers(1, &RoadNormalBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, RoadNormalBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(RoadNormal), RoadNormal, GL_STATIC_DRAW);
    
    glGenBuffers(1, &RoadIndxBuffObj);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, RoadIndxBuffObj);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(idx), idx, GL_STATIC_DRAW);
    
    glGenBuffers(1, &RoadTexBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, RoadTexBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(RoadTex), RoadTex, GL_STATIC_DRAW);
}

/* intialize the cube data */
static void initCube() {
    
    float CubePos[] = {
        -0.5, -0.5, -0.5, /*back face 5 verts :0 */
        -0.5, 0.5, -0.5,
        0.5, 0.5, -0.5,
        0.5, -0.5, -0.5,
        0.0, 0.5, -0.5,
        -0.5, -0.5, 0.5, /*front face 5 verts :5*/
        -0.5, 0.5, 0.5,
        0.5, 0.5, 0.5,
        0.5, -0.5, 0.5,
        0.0, 0.5, 0.5,
        -0.5, -0.5, 0.5, /*left face 4 verts :10*/
        -0.5, -0.5, -.5,
        -0.5, 0.5, -0.5,
        -0.5, 0.5, 0.5,
        0.5, -0.5, 0.5, /*right face 4 verts :14*/
        0.5, -0.5, -.5,
        0.5, 0.5, -0.5,
        0.5, 0.5, 0.5,
        0.0, 0.85, -0.5, /*left roof : 18 */
        0.0, 0.85, 0.5,
        -0.5, 0.5, 0.5,
        -0.5, 0.5, -0.5,
        0.0, 0.85, -0.5, /*right roof : 22 */
        0.0, 0.85, 0.5,
        0.5, 0.5, 0.5,
        0.5, 0.5, -0.5
    };
    
    float CubeNormal[] = {
        0, 0, -1,
        0, 0, -1,
        0, 0, -1,
        0, 0, -1,
        0, 0, -1,
        0, 0, 1,
        0, 0, 1,
        0, 0, 1,
        0, 0, 1,
        0, 0, 1,
        -1, 0, 0,
        -1, 0, 0,
        -1, 0, 0,
        -1, 0, 0,
        1, 0, 0,
        1, 0, 0,
        1, 0, 0,
        1, 0, 0
        /* TO DO - fill in with normals per vertex */
    };
    
    unsigned short idx[] = {0, 1, 2,  2, 3, 0,  1, 4, 2,  5, 6, 7,  7, 8, 5,  6, 9, 7,  10, 11, 12,  12, 13, 10,  14, 15, 16,  16, 17, 14};
    unsigned short roofIdx[] = {18, 19, 20, 20, 21, 18, 22, 23, 24,  24, 25, 22};
    
    g_RiboLen = 12;
    g_CiboLen = 30;
    
    glGenBuffers(1, &CubeBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, CubeBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(CubePos), CubePos, GL_STATIC_DRAW);
    
    glGenBuffers(1, &CIndxBuffObj);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, CIndxBuffObj);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(idx), idx, GL_STATIC_DRAW);
    
    glGenBuffers(1, &NormalBuffObj);
    glBindBuffer(GL_ARRAY_BUFFER, NormalBuffObj);
    glBufferData(GL_ARRAY_BUFFER, sizeof(CubeNormal), CubeNormal, GL_STATIC_DRAW);
    
    glGenBuffers(1, &RIndxBuffObj);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, RIndxBuffObj);
    glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(roofIdx), roofIdx, GL_STATIC_DRAW);
}
// 1
int getMTLinfo(string fp)
{
    // 2
    int m = 0;
    
    // 3
    // Open MTL file
    ifstream inMTL;
    inMTL.open(fp);
    if(!inMTL.good())
    {
        cout << "ERROR OPENING MTL FILE" << endl;
        exit(1);
    }
    
    // 4
    // Read MTL file
    while(!inMTL.eof())
    {
        // 5
        string line;
        getline(inMTL, line);
        string type = line.substr(0,2);
        
        if(type.compare("ne") == 0)
            m++;
    }
    
    // 6
    // Close MTL file
    inMTL.close();
    
    // 7
    return m;
}

void InitGeom() {
    
    
    initGround();
    initSky();
    initPortal();
    initCube();
    initHouseShadow();
    initRoad();

    // open an OBJ file
    std::ifstream modelFile("carmodel.obj");

    // parse the meshes from the file
    model.load(modelFile);
    
    // create VBOs from the first mesh in the file.
    // Any of the parameters can be NULL and the corresponding VBO won't be created.
    vertexCount = model.objMeshes()[0].makeVBO(&ibo, &positions, &uvs, &normals);
    // If you make IBO NULL, then the vertices will not be de-duped (you can use
    // the simpler glDrawArrays, but it will probably be more video memory).
    //vertexCount = model.objMeshes()[0].makeVBO(NULL, &positions, &uvs, &normals);
    
    scoreTally = GeometryCreator::CreateCylinder(0.1f, 0.1f, 0.0f, 20, 20);
    statusBar = GeometryCreator::CreateCylinder(0.15f, 0.15f, 20.0f, 20, 20);
    speedMeterBackground = GeometryCreator::CreateCylinder(0.5f, 0.5f, 0.0f, 20, 20);
    speedMeter = GeometryCreator::CreateCylinder(0.5f, 0.5f, 0.0f, 20, 20);
    speedArrow = GeometryCreator::CreateCube();
    treeTop = GeometryCreator::CreateCylinder(1.0f, 0.0f, 2.0, 20, 20);
    treeTrunk = GeometryCreator::CreateCylinder(0.5f, 0.5f, 1.25f, 20, 20);

}

/*function to help load the shaders (both vertex and fragment */
int InstallShader(const GLchar *vShaderName, const GLchar *fShaderName) {
    GLuint VS; //handles to shader object
    GLuint FS; //handles to frag shader object
    GLint vCompiled, fCompiled, linked; //status of shader
    
    VS = glCreateShader(GL_VERTEX_SHADER);
    FS = glCreateShader(GL_FRAGMENT_SHADER);
    
    //load the source
    glShaderSource(VS, 1, &vShaderName, NULL);
    glShaderSource(FS, 1, &fShaderName, NULL);
    
    //compile shader and print log
    glCompileShader(VS);
    /* check shader status requires helper functions */
    printOpenGLError();
    glGetShaderiv(VS, GL_COMPILE_STATUS, &vCompiled);
    printShaderInfoLog(VS);
    
    //compile shader and print log
    glCompileShader(FS);
    /* check shader status requires helper functions */
    printOpenGLError();
    glGetShaderiv(FS, GL_COMPILE_STATUS, &fCompiled);
    printShaderInfoLog(FS);
    
    if (!vCompiled || !fCompiled) {
        printf("Error compiling either shader %s or %s", vShaderName, fShaderName);
        return 0;
    }
    
    //create a program object and attach the compiled shader
    ShadeProg = glCreateProgram();
    glAttachShader(ShadeProg, VS);
    glAttachShader(ShadeProg, FS);
    
    glLinkProgram(ShadeProg);
    /* check shader status requires helper functions */
    printOpenGLError();
    glGetProgramiv(ShadeProg, GL_LINK_STATUS, &linked);
    printProgramInfoLog(ShadeProg);
    
    glUseProgram(ShadeProg);
    
    
	/* get handles to attribute data */
	h_aPosition = safe_glGetAttribLocation(ShadeProg, "aPosition");
	h_aNormal = safe_glGetAttribLocation(ShadeProg,	"aNormal");
    h_uHasATexture = safe_glGetUniformLocation(ShadeProg, "uHasATexture");

   // h_aMaterial = safe_glGetAttribLocation(ShadeProg,	"aMaterial");

    h_uProjMatrix = safe_glGetUniformLocation(ShadeProg, "uProjMatrix");
    h_uViewMatrix = safe_glGetUniformLocation(ShadeProg, "uViewMatrix");
    h_uModelMatrix = safe_glGetUniformLocation(ShadeProg, "uModelMatrix");
    h_uLightColor = safe_glGetUniformLocation(ShadeProg, "uLColor");
    h_uMatAmb = safe_glGetUniformLocation(ShadeProg, "uMat.aColor");
    h_uMatDif = safe_glGetUniformLocation(ShadeProg, "uMat.dColor");
    h_uMatSpec = safe_glGetUniformLocation(ShadeProg, "uMat.sColor");
    h_uMatShine = safe_glGetUniformLocation(ShadeProg, "uMat.shine");
    h_uLightVector = safe_glGetUniformLocation(ShadeProg, "uLightVector");
    h_aTexCoord = safe_glGetAttribLocation(ShadeProg,  "aTexCoord");
    h_uTexUnit = safe_glGetUniformLocation(ShadeProg, "uTexUnit");

    return 1;
}


/* Some OpenGL initialization */
void Initialize ()					// Any GL Init Code
{
	// Start Of User Initialization
	glClearColor (1.0f, 1.0f, 1.0f, 1.0f);
	// Black Background
 	glClearDepth (1.0f);	// Depth Buffer Setup
 	glDepthFunc (GL_LEQUAL);	// The Type Of Depth Testing
	glEnable (GL_DEPTH_TEST);// Enable Depth Testing
    
    /* texture specific settings */
    glEnable(GL_TEXTURE_2D);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    
    /* some matrix stack init */
    ModelTrans.useModelViewMatrix();
    ModelTrans.loadIdentity();
    
}


void DrawSky()
{
    glBindTexture(GL_TEXTURE_2D, 2);
    safe_glUniform1i(h_uTexUnit, 0);
    safe_glUniform1i(h_aCurTex, 2);
    
    // Position
	safe_glEnableVertexAttribArray(h_aPosition);
   	glBindBuffer(GL_ARRAY_BUFFER, SkyBuffObj);
   	safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    // Textures
    safe_glEnableVertexAttribArray(h_aTexCoord);
    glBindBuffer(GL_ARRAY_BUFFER, SkyTexBuffObj);
    safe_glVertexAttribPointer(h_aTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0);

    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, SkyIndxBuffObj);
   	glUniform3f(h_uColor, 0.7, 0.98, 0.9);
    glDrawElements(GL_TRIANGLES, g_SkyIBOLength, GL_UNSIGNED_SHORT, 0);
    safe_glDisableVertexAttribArray(h_aPosition);
}

void DrawPortal()
{
    glBindTexture(GL_TEXTURE_2D, 3);
    safe_glUniform1i(h_uTexUnit, 0);
    safe_glUniform1i(h_aCurTex, 3);
    
    // Position
	safe_glEnableVertexAttribArray(h_aPosition);
   	glBindBuffer(GL_ARRAY_BUFFER, PortalBuffObj);
   	safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    // Textures
    safe_glEnableVertexAttribArray(h_aTexCoord);
    glBindBuffer(GL_ARRAY_BUFFER, PortalTexBuffObj);
    safe_glVertexAttribPointer(h_aTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0);
    
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, PortalIndxBuffObj);
   	glUniform3f(h_uColor, 0.7, 0.98, 0.9);
    glDrawElements(GL_TRIANGLES, g_PortalIBOLength, GL_UNSIGNED_SHORT, 0);
    safe_glDisableVertexAttribArray(h_aPosition);
}

void DrawGroundPlane()
{
    safe_glUniform1i(h_uHasATexture, 1);

    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, 1);
    safe_glUniform1i(h_uTexUnit, 0);
    safe_glUniform1i(h_aCurTex, 1);
    
    // Position
    safe_glEnableVertexAttribArray(h_aPosition);
    glBindBuffer(GL_ARRAY_BUFFER, GrndBuffObj);
    safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    // Textures
    safe_glEnableVertexAttribArray(h_aTexCoord);
    glBindBuffer(GL_ARRAY_BUFFER, GTexBuffObj);
    safe_glVertexAttribPointer(h_aTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0);
    
    // bind ibo
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, GIndxBuffObj);
    glDrawElements(GL_TRIANGLES, g_GiboLen, GL_UNSIGNED_SHORT, 0);

}

void DrawRoadPlane()
{
    safe_glUniform1i(h_uHasATexture, 1);

    glBindTexture(GL_TEXTURE_2D, 0);
    safe_glUniform1i(h_uTexUnit, 0);
    safe_glUniform1i(h_aCurTex, 0);

    ModelTrans.pushMatrix();
    ModelTrans.rotate(45, vec3(0, 1, 0));
	safe_glEnableVertexAttribArray(h_aPosition);
    
   	glBindBuffer(GL_ARRAY_BUFFER, RoadBuffObj);
   	safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    safe_glEnableVertexAttribArray(h_aTexCoord);
    glBindBuffer(GL_ARRAY_BUFFER, RoadTexBuffObj);
    safe_glVertexAttribPointer(h_aTexCoord, 2, GL_FLOAT, GL_FALSE, 0, 0);
    
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, RoadIndxBuffObj);
   	glUniform3f(h_uColor, 0.0, 0.0, 0.0);
    glDrawElements(GL_TRIANGLES, g_RoadIBOlen, GL_UNSIGNED_SHORT, 0);
    safe_glDisableVertexAttribArray(h_aPosition);
    
    ModelTrans.popMatrix();
}

void DrawHouseBottom()
{
    SetMaterial(10);

    safe_glEnableVertexAttribArray(h_aPosition);
    glBindBuffer(GL_ARRAY_BUFFER, CubeBuffObj);
    safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
    safe_glEnableVertexAttribArray(h_aNormal);
    glBindBuffer(GL_ARRAY_BUFFER, NormalBuffObj);
    safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, CIndxBuffObj);
    glDrawElements(GL_TRIANGLES, g_CiboLen, GL_UNSIGNED_SHORT, 0);
    safe_glDisableVertexAttribArray(h_aPosition);

}

void DrawHouseRoof()
{
    SetModel();

    SetMaterial(4);
    safe_glEnableVertexAttribArray(h_aPosition);
    glBindBuffer(GL_ARRAY_BUFFER, CubeBuffObj);
    safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, RIndxBuffObj);
    glUniform3f(h_uColor, 0.2, 0.2, 0.2);
    glDrawElements(GL_TRIANGLES, g_RiboLen, GL_UNSIGNED_SHORT, 0);
    safe_glDisableVertexAttribArray(h_aPosition);

}

void DrawHouseShadow(int xOffset, int zOffset)
{
    ModelTrans.pushMatrix();
        ModelTrans.translate(vec3(g_tx + xOffset, g_groundY + 0.27f, zOffset - 1.0));
        SetMaterial(5);
        safe_glEnableVertexAttribArray(h_aPosition);
        glBindBuffer(GL_ARRAY_BUFFER, ShadowBuffObj);
        safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, ShadowIndxBuffObj);
        glDrawElements(GL_TRIANGLES, g_ShadowIBOLength, GL_UNSIGNED_SHORT, 0);
        safe_glDisableVertexAttribArray(h_aPosition);
    ModelTrans.popMatrix();
}

void DrawHouse(int xOffset, int zOffset)
{
    ModelTrans.pushMatrix();
        ModelTrans.translate(vec3(g_tx + xOffset, g_ty, zOffset));
        ModelTrans.pushMatrix();
            ModelTrans.translate(vec3(g_tx + xOffset, g_ty, zOffset));
            ModelTrans.scale(2, 2, 2);
            DrawHouseRoof();
        ModelTrans.popMatrix();
    DrawHouseBottom();
    ModelTrans.popMatrix();
}

void DrawTreeTop()
{
    SetModel();
    SetMaterial(12);
    safe_glEnableVertexAttribArray(h_aPosition);
    glBindBuffer(GL_ARRAY_BUFFER, treeTop->PositionHandle);
    safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    safe_glEnableVertexAttribArray(h_aNormal);
    glBindBuffer(GL_ARRAY_BUFFER, treeTop->NormalHandle);
    safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, treeTop->IndexHandle);
    glDrawElements(GL_TRIANGLES, treeTop->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
    
    safe_glDisableVertexAttribArray(h_aPosition);

}

void DrawTreeTrunk()
{
    SetModel();
    SetMaterial(11);
    safe_glEnableVertexAttribArray(h_aPosition);
    glBindBuffer(GL_ARRAY_BUFFER, treeTrunk->PositionHandle);
    safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    safe_glEnableVertexAttribArray(h_aNormal);
    glBindBuffer(GL_ARRAY_BUFFER, treeTrunk->NormalHandle);
    safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, treeTrunk->IndexHandle);
    glDrawElements(GL_TRIANGLES, treeTrunk->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
    
    safe_glDisableVertexAttribArray(h_aPosition);

}

void DrawTree(int xOffset, int zOffset)
{
    ModelTrans.pushMatrix();
    ModelTrans.translate(vec3(g_tx + xOffset, g_ty, zOffset));
    ModelTrans.pushMatrix();
    ModelTrans.translate(vec3(0, 1.24, 0));
    ModelTrans.rotate(-90, vec3(1, 0, 0));

    DrawTreeTop();

    ModelTrans.popMatrix();
    ModelTrans.rotate(-90, vec3(1, 0, 0));

    DrawTreeTrunk();
    ModelTrans.popMatrix();
}

void DrawStatusBar()
{
    ModelTrans.pushMatrix();
    ModelTrans.translate(vec3(scoreTallyDefaultPosition.x - 10, scoreTallyDefaultPosition.y + 0.25, scoreTallyDefaultPosition.z + 0.2));
    ModelTrans.rotate(90, vec3(0, 1, 0));
    
    SetModel();
    SetMaterial(5);
    safe_glEnableVertexAttribArray(h_aPosition);
    glBindBuffer(GL_ARRAY_BUFFER, statusBar->PositionHandle);
    safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    safe_glEnableVertexAttribArray(h_aNormal);
    glBindBuffer(GL_ARRAY_BUFFER, statusBar->NormalHandle);
    safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, statusBar->IndexHandle);
    glDrawElements(GL_TRIANGLES, statusBar->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
    
    safe_glDisableVertexAttribArray(h_aPosition);
    
    ModelTrans.popMatrix();
    
}
void DrawSpeedArrow()
{
    ModelTrans.pushMatrix();
    ModelTrans.translate(vec3(0.15, -0.13, 0));
    float arrowAngle = -(speedFactor * 100) - 45;
    
    if (arrowAngle < -170)
    {
        arrowAngle += 110;
    }
    
    ModelTrans.rotate(arrowAngle, vec3(0, 0, 1));
    ModelTrans.scale(0.10, 0.10, 0.0);
    ModelTrans.scale(1, 10.0, 1);

    SetModel();
    SetMaterial(14);
    safe_glEnableVertexAttribArray(h_aPosition);
    glBindBuffer(GL_ARRAY_BUFFER, speedArrow->PositionHandle);
    safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    safe_glEnableVertexAttribArray(h_aNormal);
    glBindBuffer(GL_ARRAY_BUFFER, speedArrow->NormalHandle);
    safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, speedArrow->IndexHandle);
    glDrawElements(GL_TRIANGLES, speedArrow->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
    
    safe_glDisableVertexAttribArray(h_aPosition);
    
    ModelTrans.popMatrix();
}

void DrawSpeedMeterBackground()
{
    SetModel();
    SetMaterial(13);
    safe_glEnableVertexAttribArray(h_aPosition);
    glBindBuffer(GL_ARRAY_BUFFER, speedMeterBackground->PositionHandle);
    safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    safe_glEnableVertexAttribArray(h_aNormal);
    glBindBuffer(GL_ARRAY_BUFFER, speedMeterBackground->NormalHandle);
    safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, speedMeterBackground->IndexHandle);
    glDrawElements(GL_TRIANGLES, speedMeterBackground->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
    
    safe_glDisableVertexAttribArray(h_aPosition);
}

void DrawSpeedMeter()
{
    ModelTrans.pushMatrix();
        ModelTrans.translate(speedMeterPosition);
        DrawSpeedMeterBackground();
    
        ModelTrans.scale(0.98, 0.98, 0.98);
    
        SetModel();
        SetMaterial(5);
        safe_glEnableVertexAttribArray(h_aPosition);
        glBindBuffer(GL_ARRAY_BUFFER, speedMeter->PositionHandle);
        safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
        
        safe_glEnableVertexAttribArray(h_aNormal);
        glBindBuffer(GL_ARRAY_BUFFER, speedMeter->NormalHandle);
        safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
        
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, speedMeter->IndexHandle);
        glDrawElements(GL_TRIANGLES, speedMeter->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
        
        safe_glDisableVertexAttribArray(h_aPosition);
        
        DrawSpeedArrow();
    ModelTrans.popMatrix();
}

void DrawScoreTally(int scoreNum)
{
    ModelTrans.pushMatrix();
    ModelTrans.translate(vec3(scoreTallyDefaultPosition.x - (scoreNum * 0.30), scoreTallyDefaultPosition.y + 0.05, scoreTallyDefaultPosition.z));
    
    SetModel();
    SetMaterial(2);
    safe_glEnableVertexAttribArray(h_aPosition);
    glBindBuffer(GL_ARRAY_BUFFER, scoreTally->PositionHandle);
    safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    safe_glEnableVertexAttribArray(h_aNormal);
    glBindBuffer(GL_ARRAY_BUFFER, scoreTally->NormalHandle);
    safe_glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, scoreTally->IndexHandle);
    glDrawElements(GL_TRIANGLES, scoreTally->IndexBufferLength, GL_UNSIGNED_SHORT, 0);
    
    safe_glDisableVertexAttribArray(h_aPosition);
    
    ModelTrans.popMatrix();

}



void DrawCar()
{
    ModelTrans.pushMatrix();
    ModelTrans.scale(0.3, 0.3, 0.3);
    ModelTrans.translate(carPos);
    ModelTrans.rotate(-(-90 + yawAngle * 57.2957795), vec3(0, 1, 0));
    
    SetModel();
    SetMaterial(8);
    safe_glEnableVertexAttribArray(h_aPosition);
    glBindBuffer(GL_ARRAY_BUFFER, positions);
    safe_glVertexAttribPointer(h_aPosition, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    safe_glEnableVertexAttribArray(h_aNormal);
    glBindBuffer(GL_ARRAY_BUFFER, normals);
    glVertexAttribPointer(h_aNormal, 3, GL_FLOAT, GL_FALSE, 0, 0);
    
    // If you used the IBO (generally a good idea):
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, ibo);
    glDrawElements(GL_TRIANGLES, vertexCount, GL_UNSIGNED_INT, 0);
    glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

    safe_glDisableVertexAttribArray(h_aPosition);
    ModelTrans.popMatrix();
}

float distanceBetweenTwoVectors(vec3 firstVector, vec3 secondVector)
{
    float distance = sqrt(
        pow((firstVector.x - secondVector.x), 2) +
        pow((firstVector.y - secondVector.y), 2) +
        pow((firstVector.z - secondVector.z), 2));
    
    return distance;
    
}
void CheckForCollisions()
{
    for (int houseNum = 0; houseNum < currentLevel * 2; houseNum = houseNum + 2)
    {
        // the house coordinates
        float x = houseCoords[houseNum];
        float z = houseCoords[houseNum + 1];
        float y = 1.5;
        
        vec3 housePosition = vec3(x, y, z);
        float distance = distanceBetweenTwoVectors(playerPosition, housePosition);

        if (distance < 3.0)
        {
            eyePos = defaultEyePos;
            playerPosition = defaultPlayerPos;
            carPos = defaultCarPos;
            lookAtVector = defaultLookAtVector;
            currentLevel = 1;
            speedFactor = 0.5;
        }
        
    }
}
void Draw(void)
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	//Start our shader
 	glUseProgram(ShadeProg);
    glEnable(GL_TEXTURE_2D);

    gazeVector = lookAtVector - eyePos;
    wVector = -gazeVector / length(gazeVector);
    uVector = cross(upVector, wVector) / length(cross(upVector, wVector));
    
    SetProjectionMatrix();
    SetView();
    
    glUniform3f(h_uLightColor, 1.0, 1.0, 1.0);
    glUniform3f(h_uLightVector, lightPosition.x, lightPosition.y, lightPosition.z);
    safe_glUniform1i(h_uHasATexture, 0);

    SetMaterial(0);
    SetLightModel();
    
    ModelTrans.loadIdentity();
    SetModel();
    

    safe_glUniform1i(h_uHasATexture, 1);

    SetMaterial(1);
    DrawSky();
    DrawPortal();
    SetMaterial(2);
    DrawGroundPlane();
    SetMaterial(9);
    DrawRoadPlane();

    safe_glUniform1i(h_uHasATexture, 0);
    
    scoreTallyDefaultPosition = vec3(eyePos.x + 1.5, eyePos.y + 1.5, eyePos.z + 2);
    speedMeterPosition = vec3(eyePos.x + 1.5, eyePos.y - 1.5, eyePos.z + 2);

    for (int houseNum = 0; houseNum < currentLevel * 2; houseNum = houseNum + 2)
    {
        DrawHouse(houseCoords[houseNum],
                  houseCoords[houseNum + 1]);
        
    }

    DrawCar();
    
    for (int treeNum = 0; treeNum < 80; treeNum = treeNum + 2)
    {
        // If the tree is not on the road
        if (treeCoords[treeNum] < -7 || treeCoords[treeNum] > 7)
        {
            DrawTree(treeCoords[treeNum],
                  treeCoords[treeNum + 1]);
        }
        
    }
    // For the status bar
    DrawStatusBar();
    for (int scoreNum = 0; scoreNum < currentLevel; scoreNum++)
    {
        DrawScoreTally(scoreNum);
    }
    DrawSpeedMeter();
    CheckForCollisions();
    // Disable the attributes used by our shader
    safe_glDisableVertexAttribArray(h_aPosition);
    
    //uncomment when ready
    //safe_glDisableVertexAttribArray(h_aNormal);
    
    //Disable the shader
	glUseProgram(0);
    glDisable(GL_TEXTURE_2D);
	glutSwapBuffers();
}
void ReshapeGL (int width, int height)
{
	g_width = (float)width;
	g_height = (float)height;
	glViewport (0, 0, (GLsizei)(width), (GLsizei)(height));
    
}

void Keyboard(unsigned char key, int x, int y)
{
    switch( key ) {
            /* WASD keyes effect view/camera transform */
        case 'p':
            gamePaused = !gamePaused;
            break;
        case 'a':
            if (!gamePaused)
            {
                eyePos -= speedFactor * uVector;
                lookAtVector -= speedFactor * uVector;
                carPos -= speedFactor * (float)(3.3) * uVector;
                playerPosition -= speedFactor * uVector;
            }
            break;
        case 'd':
            if (!gamePaused)
            {
                eyePos += speedFactor * uVector;
                lookAtVector += speedFactor * uVector;
                carPos += speedFactor * (float)(3.3) * uVector;
                playerPosition += speedFactor * uVector;
            }
            break;

        case 'q': case 'Q' :
            exit( EXIT_SUCCESS );
            break;
    }
    glutPostRedisplay();
}

void Idle()
{
    
}
void TimerFunction(int value)
{
    if (!gamePaused)
    {
        gazeVector = lookAtVector - eyePos;
        wVector = -gazeVector / length(gazeVector);
        uVector = cross(upVector, wVector) / length(cross(upVector, wVector));
        eyePos -= speedFactor * wVector;
        lookAtVector -= speedFactor * wVector;
        carPos -= (float)(3.3 * speedFactor) * wVector;
        playerPosition -= speedFactor * wVector;
        
        if (eyePos.z >= 45.0f)
        {
            eyePos = defaultEyePos;
            playerPosition = defaultPlayerPos;
            carPos = defaultCarPos;
            lookAtVector = defaultLookAtVector;
            speedFactor = speedFactor += 0.1;
            currentLevel = currentLevel + 1;
            printf("Advanced to Level %d!\n", currentLevel);
        }
        
    }
    glutPostRedisplay();
    glutTimerFunc(50, TimerFunction, 0);
}

int main( int argc, char *argv[] )
{
   	glutInit( &argc, argv );
   	glutInitWindowPosition( 20, 20 );
   	glutInitWindowSize( 600, 600 );
   	glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );
   	glutCreateWindow("Final Project - Lucas David");
   	glutReshapeFunc( ReshapeGL );
   	glutDisplayFunc( Draw );
	glutKeyboardFunc( Keyboard );
    glutIdleFunc( Idle );
    glutTimerFunc(50, TimerFunction, 0);
   	Initialize();
    LoadTexture((char *)"road.bmp", 0);
    LoadTexture((char *)"grass.bmp", 1);
    LoadTexture((char *)"sky.bmp", 2);
    LoadTexture((char *)"sky2.bmp", 3);


    InitTreeCoords();
	getGLversion();
	if (!InstallShader(textFileRead((char *)"vert.glsl"), textFileRead((char *)"frag.glsl"))) {
		printf("Error installing shader!\n");
		return 0;
	}
    
    if (argc == 2)
    {
        std::ifstream levelFile(argv[1]);
        loadLevel(levelFile);
        printf("Loaded level %s!\n", argv[1]);
    }
    else
    {
        printf("Loading default level!\n");
        InitHouseCoords();
    }
	InitGeom();
    printf("Welcome to Car Crasher!\n");

    
  	glutMainLoop();
    
   	return 0;
}

//routines to load in a bmp files - must be 2^nx2^m and a 24bit bmp
GLvoid LoadTexture(char* image_file, int texID) {
    
    TextureImage = (Image *) malloc(sizeof(Image));
    if (TextureImage == NULL) {
        printf("Error allocating space for image");
        exit(1);
    }
    cout << "trying to load " << image_file << endl;
    if (!ImageLoad(image_file, TextureImage)) {
        exit(1);
    }
    /*  2d texture, level of detail 0 (normal), 3 components (red, green, blue),            */
    /*  x size from image, y size from image,                                              */
    /*  border 0 (normal), rgb color data, unsigned byte data, data  */
    glBindTexture(GL_TEXTURE_2D, texID);
    glTexImage2D(GL_TEXTURE_2D, 0, 3,
                 TextureImage->sizeX, TextureImage->sizeY,
                 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage->data);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST); /*  cheap scaling when image bigger than texture */
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST); /*  cheap scaling when image smalled than texture*/
    
}

/* BMP file loader loads a 24-bit bmp file only */

/*
 * getint and getshort are help functions to load the bitmap byte by byte
 */
static unsigned int getint(FILE *fp) {
    int c, c1, c2, c3;
    
    /*  get 4 bytes */
    c = getc(fp);
    c1 = getc(fp);
    c2 = getc(fp);
    c3 = getc(fp);
    
    return ((unsigned int) c) +
    (((unsigned int) c1) << 8) +
    (((unsigned int) c2) << 16) +
    (((unsigned int) c3) << 24);
}

static unsigned int getshort(FILE *fp){
    int c, c1;
    
    /* get 2 bytes*/
    c = getc(fp);
    c1 = getc(fp);
    
    return ((unsigned int) c) + (((unsigned int) c1) << 8);
}

/*  quick and dirty bitmap loader...for 24 bit bitmaps with 1 plane only.  */

int ImageLoad(char *filename, Image *image) {
    FILE *file;
    unsigned long size;                 /*  size of the image in bytes. */
    unsigned long i;                    /*  standard counter. */
    unsigned short int planes;          /*  number of planes in image (must be 1)  */
    unsigned short int bpp;             /*  number of bits per pixel (must be 24) */
    char temp;                          /*  used to convert bgr to rgb color. */
    
    /*  make sure the file is there. */
    if ((file = fopen(filename, "rb"))==NULL) {
        printf("File Not Found : %s\n",filename);
        return 0;
    }
    
    /*  seek through the bmp header, up to the width height: */
    fseek(file, 18, SEEK_CUR);
    
    /*  No 100% errorchecking anymore!!! */
    
    /*  read the width */    image->sizeX = getint (file);
    
    /*  read the height */
    image->sizeY = getint (file);
    
    /*  calculate the size (assuming 24 bits or 3 bytes per pixel). */
    size = image->sizeX * image->sizeY * 3;
    
    /*  read the planes */
    planes = getshort(file);
    if (planes != 1) {
        printf("Planes from %s is not 1: %u\n", filename, planes);
        return 0;
    }
    
    /*  read the bpp */
    bpp = getshort(file);
    if (bpp != 24) {
        printf("Bpp from %s is not 24: %u\n", filename, bpp);
        return 0;
    }
    
    /*  seek past the rest of the bitmap header. */
    fseek(file, 24, SEEK_CUR);
    
    /*  read the data.  */
    image->data = (char *) malloc(size);
    if (image->data == NULL) {
        printf("Error allocating memory for color-corrected image data");
        return 0;
    }
    
    if ((i = fread(image->data, size, 1, file)) != 1) {
        printf("Error reading image data from %s.\n", filename);
        return 0;
    }
    
    for (i=0;i<size;i+=3) { /*  reverse all of the colors. (bgr -> rgb) */
        temp = image->data[i];
        image->data[i] = image->data[i+2];
        image->data[i+2] = temp;
    }
    
    fclose(file); /* Close the file and release the filedes */
    
    /*  we're done. */
    return 1;
}
